"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArrayStrategy = void 0;
/**
 * Maintains a sorted Array. The running-time is bad in theory, but in
 * practice Array operations are small ... assuming there isn't much data.
 *
 * The Array is stored from last entry to first: we assume queue() will be
 * the same speed either way, but this way dequeue() is O(1) instead of O(n).
 */
class ArrayStrategy {
    constructor(options) {
        var _a;
        this.options = options;
        this.data = (((_a = this.options.initialValues) === null || _a === void 0 ? void 0 : _a.slice(0)) || []);
        this.data.sort(this.comparator).reverse();
    }
    get comparator() {
        return this.options.comparator;
    }
    queue(value) {
        let pos = this._binarySearchForIndexReversed(this.data, value, this.comparator);
        this.data.splice(pos, 0, value);
    }
    dequeue() {
        return this.data.pop();
    }
    peek() {
        return this.data[this.data.length - 1];
    }
    clear() {
        this.data.length = 0;
    }
    _binarySearchForIndexReversed(array, value, comparator) {
        let low = 0;
        let high = array.length;
        while (low < high) {
            let mid = (low + high) >>> 1;
            if (comparator(array[mid], value) >= 0) { // >=, instead of the usual <
                low = mid + 1;
            }
            else {
                high = mid;
            }
        }
        return low;
    }
}
exports.ArrayStrategy = ArrayStrategy;
//# sourceMappingURL=array-strategy.js.map